package com.jsh.erp.service;

import com.baomidou.mybatisplus.extension.service.IService;
import com.jsh.erp.datasource.entities.Unit;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.http.HttpServletRequest;
import java.math.BigDecimal;
import java.util.List;

public interface UnitService extends IService<Unit> {

    Unit getUnit(long id);

    List<Unit> getUnitListByIds(String ids)throws Exception;

    /**
     * 所有单位集合
     */
    List<Unit> unitList();

    List<Unit> select(String name)throws Exception;

    /**
     * 新增单位
     * @param unit 单位
     */
    @Transactional(value = "transactionManager", rollbackFor = Exception.class)
    int insertUnit(Unit unit, HttpServletRequest request)throws Exception;

    /**
     * 修改单位
     * @param unit 单位
     */
    @Transactional(value = "transactionManager", rollbackFor = Exception.class)
    int updateUnit(Unit unit, HttpServletRequest request)throws Exception;

    /**
     * 删除单位
     * @param id 单位id
     * @param request
     */
    @Transactional(value = "transactionManager", rollbackFor = Exception.class)
    int deleteUnit(Long id, HttpServletRequest request);

    /**
     * 批量删除单位
     * @param ids 单位id字符集
     * @param request
     */
    @Transactional(value = "transactionManager", rollbackFor = Exception.class)
    int batchDeleteUnit(String ids, HttpServletRequest request) throws Exception;

    /**
     * 检查单位名称是否存在
     * @param name 单位名称
     * @return 单位数量
     */
    int checkIsNameExist(String name);

    Long getUnitIdByParam(String basicUnit, String otherUnit, BigDecimal ratio);

    BigDecimal parseStockByUnit(BigDecimal stock, Unit unitInfo, String materialUnit);

    BigDecimal parseUnitPriceByUnit(BigDecimal unitPrice, Unit unitInfo, String materialUnit);

    BigDecimal parseAllPriceByUnit(BigDecimal allPrice, Unit unitInfo, String materialUnit);

    /**
     * 批量设置状态
     * @param status 状态
     * @param ids 单位id
     */
    @Transactional(value = "transactionManager", rollbackFor = Exception.class)
    int batchSetStatus(Boolean status, String ids);

    List getUnitListByID(Long id);
}
