package com.jsh.erp.service.impl;

import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.jsh.erp.constants.BusinessConstants;
import com.jsh.erp.datasource.entities.*;
import com.jsh.erp.datasource.mappers.MaterialBatchMapper;
import com.jsh.erp.datasource.mappers.MaterialExtendMapper;
import com.jsh.erp.datasource.mappers.MaterialInitialStockMapper;
import com.jsh.erp.datasource.vo.TaskStocktakingItemVO;
import com.jsh.erp.exception.JshException;
import com.jsh.erp.exception.ServiceException;
import com.jsh.erp.query.LambdaQueryWrapperX;
import com.jsh.erp.service.*;
import com.jsh.erp.utils.DateUtils;
import com.jsh.erp.utils.RandomHelper;
import com.jsh.erp.utils.StringUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.Arrays;
import java.util.Date;
import java.util.List;

@Service
public class MaterialBatchServiceImpl extends ServiceImpl<MaterialBatchMapper,MaterialBatch> implements MaterialBatchService {
    private Logger logger = LoggerFactory.getLogger(MaterialBatchServiceImpl.class);

    @Resource
    private MaterialBatchMapper materialBatchMapper;
    @Resource
    private MaterialExtendMapper materialExtendMapper;
    @Resource
    private MaterialInitialStockMapper materialInitialStockMapper;

    @Resource
    private UserService userService;
    @Resource
    private DepotHeadService depotHeadService;
    @Resource
    private DepotItemService depotItemService;
    @Resource
    private InventoryLogService inventoryLogService;


    /**
     * 根据单据子表生成商品批次数据
     */
    @Override
    public void generateMaterialBatchByDepotItemId(DepotItem depotItem, Long supplierId) {
        //获取条码信息
        MaterialExtend materialExtend = materialExtendMapper.selectByPrimaryKey(depotItem.getMaterialExtendId());
        //创建批次信息
        MaterialBatch materialBatch = new MaterialBatch();
        //设置单据id
        materialBatch.setDepotItemId(depotItem.getId());
        //设置商品id
        materialBatch.setMaterialId(depotItem.getMaterialId());
        //设置供应商id
        materialBatch.setSupplierId(supplierId);
        //设置商品单位
        materialBatch.setCommodityUnit(depotItem.getMaterialUnit());
        //设置采购价
        materialBatch.setPurchaseDecimal(depotItem.getUnitPrice());
        //设置销售价
        materialBatch.setWholesaleDecimal(depotItem.getWholesaleDecimal());
        User user = userService.getCurrentUser();
        //创建人编码
        materialBatch.setCreateSerial(user.getLoginName());
        //创建日期
        materialBatch.setCreateTime(new Date());
        //生产日期
        materialBatch.setProductionDate(depotItem.getProductionDate());
        //保质期天数
        materialBatch.setExpiryNum(depotItem.getExpiryNum());
        //批次号
        String batchNumber = DateUtils.dateTimeNow("yyyyMMdd") + RandomHelper.getRandomStr(6);
        materialBatch.setBatchNumber(batchNumber);
        //设置库存(订单基础单位数量)
        materialBatch.setInventory(depotItem.getBasicNumber());
        //仓库id
        materialBatch.setDepotId(depotItem.getDepotId());
        //仓位货架
        materialBatch.setPosition(depotItem.getPosition());
        //条码
        materialBatch.setBarCode(materialExtend.getBarCode());
        materialBatch.setSku(depotItem.getSku());
        materialBatch.setMaterialExtendId(materialExtend.getId());
        materialBatchMapper.insert(materialBatch);
    }

    /**
     * 根据单据主表id生成商品批次数据(入库)
     * @param dhId 单据主表id
     */
    @Override
    @Transactional(value = "transactionManager", rollbackFor = Exception.class)
    public void generateMaterialBatchByDepotHeadId(Long dhId) {
        DepotHead depotHead = depotHeadService.getDepotHead(dhId);
        List<DepotItem> list = depotItemService.list(new LambdaQueryWrapperX<DepotItem>().eq(DepotItem::getHeaderId, dhId));
        //处理批次商品信息
        if (BusinessConstants.DEPOTHEAD_TYPE_IN.equals(depotHead.getType())) {
            for (DepotItem depotItem : list) {
                //表单入库,新增批次商品信息
                this.generateMaterialBatchByDepotItemId(depotItem, depotHead.getOrganId());
                //更新当前库存
                depotItemService.updateCurrentStock(depotItem);
            }
            depotHeadService.update(new UpdateWrapper<DepotHead>().set("status",2).eq("id",dhId));
        } else if (BusinessConstants.DEPOTHEAD_TYPE_OUT.equals(depotHead.getType())) {
            for (DepotItem depotItem : list) {
                //表单出库，修改商品库存
                this.handleMaterialBatchByDepotItemId(depotItem.getId());
            }
            depotHeadService.update(new UpdateWrapper<DepotHead>().set("status",2).eq("id",dhId));
        }
        depotHeadService.sendMsg(dhId);
    }

    /**
     * 根据单据子表id处理商品批次数据(出库)
     */
    @Override
    @Transactional(value = "transactionManager", rollbackFor = Exception.class)
    public void handleMaterialBatchByDepotItemId(Long diId) {
        DepotItem depotItem = depotItemService.getDepotItem(diId);
        //获取商品期初库存
        MaterialInitialStock mis = materialInitialStockMapper.selectOne(new LambdaQueryWrapperX<MaterialInitialStock>()
                .eq(MaterialInitialStock::getMaterialId,depotItem.getMaterialId())
                .eq(MaterialInitialStock::getDepotId,depotItem.getDepotId())
                .eq(MaterialInitialStock::getDeleteFlag,"0"));
        //根据单据商品id查询商品批次数据
        List<MaterialBatch> list = materialBatchMapper.getMaterialBatchByMaterialId(depotItem.getMaterialId(),depotItem.getDepotId());
        //根据单据子表基础单位数量减去批次库存
        BigDecimal basicNumber = depotItem.getBasicNumber();
        if (mis != null && mis.getNumber() != null && mis.getNumber().compareTo(basicNumber) >= 0){
            //初期库存足够，扣除期初库存
            BigDecimal inventory = mis.getNumber().subtract(basicNumber);
            mis.setNumber(inventory);
            materialInitialStockMapper.updateByPrimaryKeySelective(mis);
        }else {
            //初期库存不为空，先扣除期初，再从批次扣除
            if (mis != null && mis.getNumber() != null){
                basicNumber = basicNumber.subtract(mis.getNumber());
                mis.setNumber(BigDecimal.ZERO);
                materialInitialStockMapper.updateByPrimaryKeySelective(mis);
            }
            for (MaterialBatch materialBatch : list) {
                if (materialBatch.getInventory().compareTo(basicNumber) >= 0){
                    //批次库存足够，扣除库存，结束循环
                    BigDecimal inventory = materialBatch.getInventory().subtract(basicNumber);
                    materialBatch.setInventory(inventory);
                    updateInventory("出库",diId,materialBatch);
                    basicNumber = BigDecimal.ZERO;
                    break;
                }else {
                    //库存不足,扣除当前批次库存,继续循环
                    basicNumber = basicNumber.subtract(materialBatch.getInventory());
                    materialBatch.setInventory(BigDecimal.ZERO);
                    updateInventory("出库",diId,materialBatch);
                }
            }
            if (basicNumber.compareTo(BigDecimal.ZERO) > 0){
                throw new ServiceException("商品库存不足");
            }
        }
    }

    /**
     * 修改商品批次库存
     * @param type 操作记录
     * @param materialBatch 商品批次信息
     */
    @Override
    @Transactional(value = "transactionManager", rollbackFor = Exception.class)
    public synchronized void updateInventory(String type, Long id, MaterialBatch materialBatch) {
        if (materialBatch.getInventory() != null){
            //获取修改前库存
            int originalStock = materialBatchMapper.selectOne("id",materialBatch.getId()).getInventory().intValue();
            User user = userService.getCurrentUser();
            InventoryLog log = new InventoryLog();
            log.setUpdateUser(user.getId());
            log.setUpdateTime(new Date());
            log.setMaterialId(materialBatch.getMaterialId());
            log.setMaterialExtendId(materialBatch.getId());
            log.setOriginalStock(originalStock);
            log.setCurrentStock(materialBatch.getInventory().intValue());
            log.setItemId(id);
            log.setType(type);
            inventoryLogService.save(log);
            update(new UpdateWrapper<MaterialBatch>().set("inventory",materialBatch.getInventory()).eq("id",materialBatch.getId()));
            depotItemService.updateCurrentStockFun(materialBatch.getMaterialId(),materialBatch.getDepotId());
        }
    }

    /**
     * 根据仓库id和商品id查询商品批次库存
     * @param depotList 仓库id
     * @param mid       商品id
     */
    @Override
    public BigDecimal getInventorySumByDepotAndMid(List<Long> depotList, Long mid) {
        return materialBatchMapper.getInventorySumByDepotAndMid(depotList,mid);
    }

    /**
     * 获取批次号字符
     */
    @Override
    public List<MaterialVo4Unit> findBySelectWithBarCode(Long categoryId, String q, String standardOrModel, String color, String brand, String mfrs, String enableSerialNumber, String enableBatchNumber, Integer offset, Integer rows, Long depotId) throws Exception {
        List<MaterialVo4Unit> list =null;
        try{
            if(StringUtil.isNotEmpty(q)) {
                q = q.replace("'", "");
                q = q.trim();
            }
            list=  materialBatchMapper.findBySelectWithBarCode(q, standardOrModel, color, brand, mfrs,
                    enableSerialNumber, enableBatchNumber, offset, rows,depotId);
        }catch(Exception e){
            JshException.readFail(logger, e);
        }
        return list;
    }

    /**
     * 根据商品条码和仓库id获取商品批次信息
     * @param barCodes 商品条码字符集合
     */
    @Override
    public List<MaterialBatch> findBySelectWithBarCode(String barCodes,Long depotId) {
        List<String> barCodeList = null;
        if (barCodes != null && !barCodes.isEmpty()){
            barCodeList = Arrays.asList(barCodes.split(","));
        }
        List<MaterialBatch> list = materialBatchMapper.selectList(new LambdaQueryWrapperX<MaterialBatch>().eq(MaterialBatch::getDepotId,depotId).inIfPresent(MaterialBatch::getSku,barCodeList).gt(MaterialBatch::getInventory,BigDecimal.ZERO));
        return list;
    }

    /**
     * 根据批次号数组查询商品批次信息
     * @param batchNumbers 批次号数组
     */
    @Override
    public List<TaskStocktakingItemVO> getMaterialByBatchNumber(String[] batchNumbers) {
        return materialBatchMapper.getMaterialByBatchNumber(batchNumbers);
    }

}
